#!/usr/bin/env python
# -*- coding: UTF-8 -*-

# Copyright 2011 (C) Raster Software Vigo (Sergio Costas)

# This file is part of EasyWall
#
# EasyWall is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# EasyWall is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


# This file contains easywall's core. It is independent of the widget
# system, so it should be used both for GTK and QT.

import xml.sax
import sys
import os
import stat
import subprocess

class xml_write:
	
	""" This class allows to easily create the XML files """
	
	def __init__(self,filename):
		
		self.file=open(filename,"w")
		self.file.write("<?xml version='1.0' encoding='UTF-8'?>\n")

		
	def write_tag(self,tag,autoclose,newline,tabs=0,attrs=None):
		
		for count in range(tabs):
			self.file.write('\t')
		
		self.file.write('<'+tag)
		if attrs!=None:
			for element in attrs:
				self.file.write(' '+element+'="'+self.expand(attrs[element])+'"')
		if autoclose:
			self.file.write(' /')
		self.file.write('>')
		if newline:
			self.file.write('\n')


	def write_text(self,text):
		self.file.write(self.expand(text))
		

	def close(self):
		
		self.file.close()


	def expand(self,text):
		
		text=text.replace('&','&amp;')
		text=text.replace('<','&lt;')
		text=text.replace('>','&gt;')
		text=text.replace('"','&quot;')
		text=text.replace("'",'&apos;')
		return text
	

class easywall_read_xml(xml.sax.handler.ContentHandler):
	
	""" This class reads the XML files with the port definitions """
	
	def read_xml(self,filename):
		
		self.mode=""
		self.locale=None
		self.port_type=False
		self.port_value=False
		self.init_current_port()
		self.current_program={}
		self.current_program["category"]=""
		self.current_program["names"]={}
		self.current_program["unique-id"]=""
		self.current_program["active"]=False
		self.current_program["files"]=[]
		self.current_program["ports"]=[]
		self.current_program["modules"]=[]
		
		self.header=False
		
		self.parser=xml.sax.make_parser()
		self.parser.setContentHandler(self)
		
		self.ports_failed=False
		
		try:
			self.read_file(filename)
			if self.current_program==None:
				print "Error en "+str(filename)
		except:#xml.sax._exceptions.SAXParseException,(errno,strerror):
			self.current_program=None
			print "Error while parsing "+str(filename)
			#print "%s %s" % (errno,strerror)
			print


	def read_file(self,filename):
		
		xml_file=open(filename,"r")
		self.parser.parse(xml_file)
		xml_file.close()
		if self.ports_failed:
			self.current_program=None
		elif (self.current_program["category"]==""):
			self.current_program=None
		elif (self.current_program["unique-id"]==""):
			self.current_program=None
		elif (len(self.current_program["names"])==0):
			self.current_program=None
		elif (len(self.current_program["ports"])==0) and (len(self.current_program["modules"])==0):
			self.current_program=None
	
	
	def init_current_port(self):
		self.current_port={"icmp":False,"tcp":True,"id":"","base":"","value":-1,"range":-1,"name":{},"icmp_type":-1,"icmp_code":-1}
	
	
	def startElement(self,name,attrs):
		
		if name=="easywall":
			self.header=True
			
		if self.header==False:
			return
		
		if (self.mode=="") and (name!="easywall"):
			return
		
		self.mode=name
		if name=="port":
			self.doing_port=True
			self.init_current_port()
			if attrs.get("editable","")=="yes":
				self.current_port["editable"]=True
			else:
				self.current_port["editable"]=False
			return
		
		if (name=="name") or (name=="port-name"):
			self.locale=attrs.get("locale")

		
	def endElement(self,name):
		
		self.locale=None
		
		if self.mode=="":
			return
		
		if name=="easywall":
			self.mode=""
			self.header=False
			return
		
		if name=="port":
			if (self.current_port["icmp"]==False):
				if (self.current_port["value"]==-1):
					print "VALUE field incorrect"
					self.ports_failed=True
				elif ((self.current_port["editable"]) and (self.current_port["id"]=="")):
					print "EDITABLE port withoud ID"
					self.ports_failed=True
				elif ((self.current_port["editable"]) and (self.current_port["base"]!="")):
					self.current_port["editable"]=False
			else:
				self.current_port["editable"]=False
				if (self.current_port["icmp_type"]==-1):
					print "ICMP without TYPE"
					self.ports_failed=True

			self.current_program["ports"].append(self.current_port)
			self.init_current_port()
			return


	def characters(self,chars):
		
		if (self.mode=="") or (self.header==False):
			return
		
		while (chars!="") and ((chars[0]==" ") or (chars[0]=="\n") or (chars[0]=="\t")):
			chars=chars[1:]
		
		while (chars!="") and ((chars[-1]==" ") or (chars[-1]=="\n") or (chars[-1]=="\t")):
			chars=chars[:-1]
		
		if chars=="":
			return
		
		if self.mode=="category":
			self.current_program["category"]=chars
		elif self.mode=="unique-id":
			self.current_program["unique-id"]=chars
		elif self.mode=="check-file":
			self.current_program["files"].append(chars)
		elif self.mode=="module":
			self.current_program["modules"].append(chars)
		elif self.mode=="name":
			if self.locale==None:
				locale="DEFAULT"
			else:
				locale=self.locale
			self.current_program["names"][locale]=chars
			self.locale=None
			
		elif self.mode=="type":
			self.current_port["icmp"]=False
			if chars=="icmp":
				self.current_port["icmp"]=True
				self.current_port["tcp"]=False
			elif chars=="tcp":
				self.current_port["tcp"]=True
			else:
				self.current_port["tcp"]=False
		elif self.mode=="id":
			self.current_port["id"]=chars
		elif self.mode=="base":
			self.current_port["base"]=chars
		elif self.mode=="value":
			try:
				value=int(chars)
			except:
				value=-1
			self.current_port["value"]=value
		elif self.mode=="range":
			try:
				range=int(chars)
				if range<2:
					range=-1
			except:
				range=-1
			self.current_port["range"]=range
		elif self.mode=="port-name":
			if self.locale==None:
				locale="DEFAULT"
			else:
				locale=self.locale
			self.current_port["name"][locale]=chars
			self.locale=None
		elif self.mode=="icmp_type":
			try:
				icmp_type=int(chars)
			except:
				icmp_type=-1
			if icmp_type<-1:
				icmp_type=-1
			if icmp_type>255:
				icmp_type=255
			self.current_port["icmp_type"]=icmp_type
		elif self.mode=="icmp_code":
			try:
				icmp_code=int(chars)
			except:
				icmp_code=-1
			if icmp_code<-1:
				icmp_code=-1
			if icmp_code>255:
				icmp_code=255
			self.current_port["icmp_code"]=icmp_code
	

class easywall_read_modifiers(xml.sax.handler.ContentHandler):
	
	""" This class reads the XML files with the modifiers for each port """
	
	def __init__(self):
		
		self.modifiers=[]
		self.mode=""
		self.reset_program(None)
		
		self.parser=xml.sax.make_parser()
		self.parser.setContentHandler(self)
		
		self.header=False
		
		try:
			xml_file=open("/etc/easywall/modifiers.xml","r")
			self.parser.parse(xml_file)
			xml_file.close()
		except:
			pass
		
	def reset_program(self,id):
		
		self.current_program={"id":id,"ports":{}}

	
	def startElement(self,name,attrs):
		
		if name=="easywall-modifiers":
			self.header=True
			
		if self.header==False:
			return
		
		if name=="program":
			id=attrs.get("unique-id")
			if id!=None:
				self.reset_program(id)

		elif name=="port-modify":
			id=attrs.get("id")
			port=attrs.get("port")
			range=attrs.get("range")
			if (range==None) or (range<1):
				range=1
			if (id!=None) and (port!=None):
				try:
					self.current_program["ports"][id]=[int(port),int(range)]
				except:
					pass


	def endElement(self,name):

		if name=="easywall-modifiers":
			self.header=False

		if name=="program":
			if (self.current_program["id"]!=None) and (len(self.current_program["ports"])!=0):
				self.modifiers.append(self.current_program)
			self.reset_program(None)


class easywall_read_active(xml.sax.handler.ContentHandler):

	""" This class reads the currently active program definitions """

	def __init__(self):
		
		self.active=[]
		
		self.parser=xml.sax.make_parser()
		self.parser.setContentHandler(self)
		
		self.header=False
		
		try:
			xml_file=open("/etc/easywall/active.xml","r")
			self.parser.parse(xml_file)
			xml_file.close()
		except:
			pass

	
	def startElement(self,name,attrs):
		
		if name=="easywall-active":
			self.header=True
			
		if self.header==False:
			return
		
		if name=="active":
			id=attrs.get("unique-id")
			if id!=None:
				self.active.append(id)

	def endElement(self,name):
		
		if name=="easywall-active":
			self.header=False


class easywall_read_templates(xml.sax.handler.ContentHandler):

	""" This class reads the templates used to create the firewall script """

	def __init__(self):
		
		self.template_base=""
		self.template_line=""
		self.template_icmp=""
		self.template_module=""

		self.header=False

		self.tag=""

		self.parser=xml.sax.make_parser()
		self.parser.setContentHandler(self)
		
		done=False
		
		try:
			xml_file=open("/etc/easywall/template.xml","r")
			done=True
		except:
			pass
		
		if done==False:
			try:
				xml_file=open("/etc/easywall/template_base.xml","r")
				done=True
			except:
				pass
		
		if done:
			try:
				self.parser.parse(xml_file)
				xml_file.close()
			except:
				pass

	
	def startElement(self,name,attrs):
		
		if name=="easywall-templates":
			self.header=True
			
		if self.header==False:
			return
		
		if name=="base":
			self.tag="base"
		elif name=="line":
			self.tag="line"
		elif name=="icmp":
			self.tag="icmp"
		elif name=="module":
			self.tag="module"
		else:
			self.tag=""
			
			
	def endElement(self,name):
		
		if name=="easywall-templates":
			self.header=False
		
		self.tag=""
		

	def characters(self,chars):

		if self.header==False:
			return
			
		if self.tag=="base":
			self.template_base+=chars
		elif self.tag=="line":
			self.template_line+=chars
		elif self.tag=="icmp":
			self.template_icmp+=chars
		elif self.tag=="module":
			self.template_module+=chars


class easywall_core:
	
	def __init__(self,locale):
		
		""" This class implements all the logic in EasyWall, so the window class can
			use it in a portable way """

		# ensure that the directories exists
		try:
			os.mkdir("/etc/easywall")
			os.mkdir("/etc/easywall/xml")
		except:
			pass

		if locale.find("_")==-1:
			self.locale=locale[:2]
			self.hlocale=None
		else:
			self.locale=locale[:5]
			self.hlocale=locale[:2]

		self.output=""

		self.read_port_definition_files()
		self.read_template()


	def get_output(self):
		
		return self.output


	def get_user_defined_category(self):
		
		""" Returns the string used for the "User defined" category """
		
		return _("User defined")


	def get_program_list(self):
		
		""" This method returns a list of dictionaries with all the programs to be shown,
			its cathegory and its unique-id """

		program_list=[]
		for element in self.ports:
			if element["remove"]:
				continue
			name=self.get_translated_string(element["names"])
			type=element["category"].lower()
			if type=="service":
				category=_("Services")
			elif type=="icmp":
				category=_("ICMP")
			elif type=="p2p":
				category=_("P2P")
			elif type=="file_transfer":
				category=_("File Transfer")
			elif type=="chat":
				category=_("Chat")
			elif type=="videoconference":
				category=_("Videoconference")
			elif type=="games":
				category=_("Games")
			elif type=="user_defined":
				category=_("User defined")
			else:
				category=_("Other")
			
			unique_id=element["unique-id"]
			active=element["active"]
			program_list.append({"unique-id":unique_id,"category":category,"name":name,"active":active})
		
		return program_list


	def change_user_defined_program(self,unique_id,name,port_list):
		
		""" Allows to change the ports and visible name used in a user-defined program.
			PORT_LIST is a list with a variable number of tuples, each one containing
			first the port number, and then TRUE when the port is TCP or FALSE when is
			UDP.
			
			If unique_id is None, it will create a new program and assign an unique id """
		
		if unique_id!=None:
		
			found=None
			for element in self.ports:
				if element["unique-id"]==unique_id:
					found=element
					break
		
			if found==None:
				return -1 # the unique-id is not found in the current list
		
			if found["category"].lower()!="user_defined":
				return -2 # found, but it's not an user-defined program
		else:
			
			found={"category":"user_defined","files":[],"remove":False,"active":False,"ports":[]}
			counter=0
			while(True):
				notfound=True
				for element in self.ports:
					if element["unique-id"]=="udef"+str(counter):
						notfound=False
						break
				if notfound:
					break
				counter+=1
			found["unique-id"]="udef"+str(counter)
			found["modules"]=""
			self.ports.append(found)
		
		found["names"]={"DEFAULT":name}
		newport=[]
		for element in port_list:
			newport.append({"name":"", "editable":False,"value":element[0],"range":-1,"base":"","tcp":element[1],"id":"","icmp":False,"icmp_type":-1,"icmp_code":-1})
		found["ports"]=newport


	def get_program_info(self,unique_id):
		
		""" This method returns the program's info with the specified unique id """
		
		for element in self.ports:
			if element["unique-id"]!=unique_id:
				continue
			
			if element["remove"]:
				continue
			name=self.get_translated_string(element["names"])
			type=element["category"].lower()
			if type=="server":
				category=_("Servers")
			elif type=="icmp":
				category=_("ICMP")
			elif type=="p2p":
				category=_("P2P")
			elif type=="file_transfer":
				category=_("File Transfer")
			elif type=="chat":
				category=_("Chat")
			elif type=="videoconference":
				category=_("Videoconference")
			elif type=="games":
				category=_("Games")
			elif type=="user_defined":
				category=_("User defined")
			else:
				category=_("Other")
			
			unique_id=element["unique-id"]
			active=element["active"]
			return ({"unique-id":unique_id,"category":category,"name":name,"active":active})

		return None


	def set_program_status(self,unique_id,status):
		
		""" Sets the status (TRUE=open ports, FALSE=close ports) for the specified program """
		
		new_active=[]
		if status==False: # remove it from the list
			for element in self.active:
				if element!=unique_id:
					new_active.append(element)
			self.active=new_active

		else: # add it to the list
			if 0==self.active.count(unique_id):
				self.active.append(unique_id)

		for element in self.ports:
			if element["unique-id"]==unique_id:
				element["active"]=status


	def get_program_status(self,unique_id):

		""" Returns the status (True=open ports, False=close ports) for the
			specified program """
		
		for element in self.ports:
			if element["unique-id"]==unique_id:
				return element["active"]
			
		return False
	
	
	def get_program_modules(self,unique_id):
		
		""" Gets the modules to be loaded for a program """
		
		print unique_id
		program=None
		for element in self.ports:
			if element["unique-id"]==unique_id:
				program=element
				break
			
		if program==None:
			return None # not found
		
		return program["modules"]
	
	
	def get_program_ports(self,unique_id):
		
		""" Gets the ports for a program, using both the original template and
			the modified one when needed """
		
		ports=[]
		
		program=None
		for element in self.ports:
			if element["unique-id"]==unique_id:
				program=element
				break
			
		if program==None:
			return None # not found
		
		modified=None
		for element in self.modifiers:
			if element["id"]==unique_id:
				modified=element["ports"]
		
		for port in program["ports"]:
			current_port={}
			if port["editable"] and (modified!=None):
				for element in modified:
					if port["id"]==element:
						port["value"]=modified[element][0] # modify the port if needed
						if port["range"]!=-1:
							port["range"]=modified[element][1]
			
			if (port["base"]=="") or (port["icmp"]):
				current_port["id"]=port["id"]
				current_port["value"]=port["value"]
				if port["icmp"]:
					current_port["editable"]=False
				else:
					current_port["editable"]=port["editable"]
				current_port["range"]=port["range"]
				current_port["tcp"]=port["tcp"]
				current_port["name"]=self.get_translated_string(port["name"])
				current_port["icmp"]=port["icmp"]
				current_port["icmp_type"]=port["icmp_type"]
				current_port["icmp_code"]=port["icmp_code"]
				ports.append(current_port)
		
		for port in program["ports"]:
			if port["icmp"]:
				continue
			current_port={}
			if (port["base"]!=""):
				port_base=None
				for element in ports:
					if element["id"]==port["base"]:
						port_base=element
				if port_base==None:
					continue # base port not found
				current_port["icmp"]=False
				current_port["id"]=port["id"]
				current_port["value"]=port_base["value"]+port["value"]
				current_port["editable"]=False
				current_port["range"]=port["range"]
				current_port["tcp"]=port["tcp"]
				current_port["name"]=self.get_translated_string(port["name"])
				ports.append(current_port)
		return ports
	
	
	def set_port_value(self,unique_id,port_id,value,range):
	
		""" Modifies an editable port (specified by port_id) in the program specified
			by unique_id """
		
		found=False
		if value>65535:
			value=65535
		if value<1:
			value=1
		if range<1:
			range=1
		for element in self.ports:
			if element["unique-id"]==unique_id:
				for port in element["ports"]:
					if (port["id"]==port_id):
						if ((port["editable"]) and (port["icmp"]==False)):
							found=True
						break

		if found==False:
			return

		found=None
		for element in self.modifiers:
			if element["id"]==unique_id:
				found=element

		if found==None:
			self.modifiers.append({"id":unique_id,"ports":{port_id:[value,range]}})
			return
		
		found["ports"][port_id]=[value,range]


	def get_module_list(self):
		
		""" Returns the list of modules to be loaded by the firewall """

		modules=[]
		programs=self.get_program_list()
		for element in programs:
			if element["active"]==False:
				continue
			module_list=self.get_program_modules(element["unique-id"])
			for module in module_list:
				if modules.count(module)==0:
					modules.append(module)
		
		return modules


	def get_port_list(self):
		
		""" Returns the list of ports to open.
			It returns a tupla with three lists: the first one are the TCP ports,
			the second one the UDP ports, and the third one contains two-element
			tuplas with the ICMP ports (first element is the type, and second
			element is the code, or -1 when has to use all codes """


		tcp_ports=[]
		udp_ports=[]
		icmp_ports=[]
		icmp_tmp=[]
		
		programs=self.get_program_list()
		
		for element in programs:
			if element["active"]==False:
				continue
			
			port_list=self.get_program_ports(element["unique-id"])
			for port in port_list:
				if port["icmp"]:
					if port["icmp_code"]==-1:
						icmp_ports.append((port["icmp_type"],-1))
						icmp_tmp.append(port["icmp_type"])
					else:
						icmp_ports.append((port["icmp_type"],port["icmp_code"]))
				else:
					if port["range"]==-1:
						vrange=1
					else:
						vrange=port["range"]
					for loop in range(vrange):
						if port["tcp"]:
							if 0==tcp_ports.count(port["value"]+loop):
								tcp_ports.append(port["value"]+loop)
						else:
							if 0==udp_ports.count(port["value"]+loop):
								udp_ports.append(port["value"]+loop)

		icmp_ports_final=[]
		for element in icmp_ports:
			if element[1]!=-1:
				if icmp_tmp.count(element[0])==0:
					icmp_ports_final.append(element)
			else:
				if icmp_ports_final.count(element)==0:
					icmp_ports_final.append(element)

		return (tcp_ports,udp_ports,icmp_ports_final)


	def remove_user_defined_program(self,unique_id):
		
		""" Removes an user-defined program from the list """

		found=False
		newports=[]
		for element in self.ports:
			if (element["unique-id"]==unique_id) and (element["category"]=="user_defined"):
				found=True
			else:
				newports.append(element)
		if found==False:
			return -1 # not found
		else:
			self.ports=newports
			return 0
		
			

	def create_firewall(self):
		
		""" Creates the firewall script but doesn't relaunchs it """
		
		(tcp_ports,udp_ports,icmp_ports)=self.get_port_list()
		
		iptables=""
		
		tmp=self.template_icmp[:]
		if (tmp=="") or (tmp[-1]!="\n"):
			tmp+="\n"
		for element in icmp_ports:
			if (element[1]==-1):
				replacement=str(element[0])
			else:
				replacement=str(element[0])+"/"+str(element[1])
			iptables+=tmp.replace("[TYPE]",replacement)
		iptables+="\n"

		
		tmp=self.template_line.replace("[TYPE]","tcp")
		if tmp[-1]!="\n":
			tmp+="\n"
		for element in tcp_ports:
			iptables+=tmp.replace("[PORT]",str(element))
		iptables+="\n"

		tmp=self.template_line.replace("[TYPE]","udp")
		if tmp[-1]!="\n":
			tmp+="\n"
		for element in udp_ports:
			iptables+=tmp.replace("[PORT]",str(element))
		
		iptables+="\n"
		iptables=iptables.replace("[\\N]","\n")
		
		module_list=self.get_module_list()
		
		modules=""
		tmp=self.template_module[:]
		if (tmp=="") or (tmp[-1]!="\n"):
			tmp+="\n"
		for element in module_list:
			modules+=tmp.replace("[NAME]",element)
		modules=modules.replace("[\\N]","\n")
		
		firewall=self.template_base.replace("[IPTABLES_LINES]",iptables)
		firewall=firewall.replace("[MODULES_LINES]",modules)

		try:
			fire_file=open("/etc/init.d/easywall.sh","w")
			fire_file.write(firewall)
			fire_file.close()
			os.chmod("/etc/init.d/easywall.sh",stat.S_IREAD | stat.S_IWRITE | stat.S_IEXEC)
		except:
			return -1

		try:
			for element in range(4):
				os.symlink("/etc/init.d/easywall.sh","/etc/rc"+str(element+1)+".d/S99easywall")
		except:
			return -1

		return 0
	
	
	def store_configuration(self):
		
		""" Saves the current status (modifiers and active programs) and the
			new user-defined programs.
			Returns 0 if all went fine, or -1 if there was an error """
		
		try:
			writer=xml_write("/etc/easywall/modifiers.xml")
			writer.write_tag("easywall-modifiers",False,True)
			for element in self.modifiers:
				writer.write_tag("program",False,True,1,{"unique-id":element["id"]})
				lports=element["ports"]
				for port in lports:
					if lports[port][1]<1:
						writer.write_tag("port-modify",True,True,2,{"id":port,"port":str(lports[port][0])})
					else:
						writer.write_tag("port-modify",True,True,2,{"id":port,"port":str(lports[port][0]),"range":str(lports[port][1])})
				writer.write_tag("/program",False,True,1)
			writer.write_tag("/easywall-modifiers",False,True)
			writer.close()
		except:
			return -1
		
		try:
			writer=xml_write("/etc/easywall/active.xml")
			writer.write_tag("easywall-active",False,True)
			for element in self.active:
				writer.write_tag("active",True,True,1,{"unique-id":element})
			writer.write_tag("/easywall-active",False,True)
			writer.close()
		except:
			return -1
		
		filelist=os.listdir("/etc/easywall/xml")
		for element in filelist:
			name=os.path.join("/etc/easywall/xml",element)
			os.remove(name)

		try:
			count=0
			for element in self.ports:
				if element["category"]!="user_defined":
					continue
				writer=xml_write("/etc/easywall/xml/user_defined"+str(count)+".xml")
				writer.write_tag("easywall",False,True)
				writer.write_tag("category",False,False,1)
				writer.write_text("user_defined")
				writer.write_tag("/category",False,True,0)
				
				writer.write_tag("unique-id",False,False,1)
				writer.write_text(element["unique-id"])
				writer.write_tag("/unique-id",False,True,0)
				
				writer.write_tag("name",False,False,1)
				writer.write_text(element["names"]["DEFAULT"])
				writer.write_tag("/name",False,True,0)
				
				for element2 in element["ports"]:
					writer.write_tag("port",False,True,1)
					writer.write_tag("type",False,False,2)
					if element2["tcp"]:
						writer.write_text("tcp")
					else:
						writer.write_text("udp")
					writer.write_tag("/type",False,True,0)
					writer.write_tag("value",False,False,2)
					writer.write_text(str(element2["value"]))
					writer.write_tag("/value",False,True,0)
					writer.write_tag("/port",False,True,1)
				writer.write_tag("/easywall",False,False)
				writer.close()
				count+=1
		except:
			return -1

		return 0


	def read_template(self):
		
		""" Reads the templates for the firewall from /etc/easywall/template.xml, or
			from /etc/easywall/template_base.xml when the former doesn't exist """
		
		tpl=easywall_read_templates()
		self.template_base=tpl.template_base
		self.template_line=tpl.template_line
		self.template_icmp=tpl.template_icmp
		self.template_module=tpl.template_module


	def get_templates(self):
		
		""" Returns a tupla with the firewall templates """
		
		return ((self.template_base, self.template_line, self.template_icmp, self.template_module))


	def set_templates(self,template_base,template_line,template_icmp,template_module):
		
		""" Changes the firewall templates """
		
		self.template_base=template_base
		self.template_line=template_line
		self.template_icmp=template_icmp
		self.template_module=template_module


	def write_template(self):
		
		""" Writes the templates for the firewall """
		
		try:
			writer=xml_write("/etc/easywall/template.xml")
			writer.write_tag("easywall-templates",False,True)
			writer.write_tag("line",False,False,1)
			writer.write_text(self.template_line)
			writer.write_tag("/line",False,True,0)
			writer.write_tag("icmp",False,False,1)
			writer.write_text(self.template_icmp)
			writer.write_tag("/icmp",False,True,0)
			writer.write_tag("module",False,False,1)
			writer.write_text(self.template_module)
			writer.write_tag("/module",False,True,0)
			writer.write_tag("base",False,False,1)
			writer.write_text(self.template_base)
			writer.write_tag("/base",False,True,0)
			writer.write_tag("/easywall-templates",False,True)
			writer.close()
		except:
			return -1
		
		return 0


	def restore_templates(self):
		
		""" Restores the original templates """
		
		try:
			os.remove("/etc/easywall/template.xml")
		except:
			pass
		self.read_template()
	
	
	def get_firewall_status(self):
		
		""" Returns True if the firewall is active, and False if it's inactive """
		
		s=os.popen("iptables -L -n")
		status=False
		
		while True:
			line=s.readline()
			
			if (line=="") or (line==None):
				break
			
			if (line.find("Chain INPUT")!=-1):
				if line.find("DROP")!=-1:
					status=True
				else:
					status=False

		return status					


	def set_firewall_status(self,status):
		
		""" Calls the firewall with the specified status (stop, start or restart).
			Returns the error code. """
		
		s=subprocess.Popen("/etc/init.d/easywall.sh "+status,stdout=subprocess.PIPE,stderr=subprocess.STDOUT,shell=True)

		line="g"
		self.output=""
		while (line!="") and (line!=None):
			line=s.stdout.readline()
			if line!=None:
				self.output+=line
		
		return (s.wait())
		


	def get_firewall_install(self):
		
		""" Returns False if the firewall script still doesn't exists, or True if it
			already exists """
			
		installed=False
		try:
			f=open("/etc/init.d/easywall.sh","r")
			f.close()
			installed=True
		except:
			pass
		
		return installed


	def get_translated_string(self,strings):
		
		""" Returns a translated string, based on the current locale, taken from the STRINGS dictionary.
			If there isn't a translated string for the current locale, will try to use one in a higher level,
			and only if there's no string available will return the DEFAULT one.
			e.g.: if LOCALE is PT_BR, will try to return the translation for PT_BR. If it doesn't exists, will
			try with the translation for PT; if still no matches, will return the DEFAULT one.
			And if the DEFAULT doesn't exists, will return the first one.
			If the dictionary is empty, will return NONE """

		if (len(strings)==0):
			return None

		if (strings.has_key(self.locale)):
			return strings[self.locale]
		
		if (self.hlocale!=None):
			if (strings.has_key(self.hlocale)):
				return strings[self.hlocale]
			
		if strings.has_key("DEFAULT"):
			return strings["DEFAULT"]
		
		return strings[strings.keys()[0]]


	def read_port_definition_files(self):
		
		""" This method reads all the XML port definition files in all available directories
			(/usr/share/easywall/xml,/usr/local/share/easywall/xml and ./xml) and creates
			a list with them.

			The files created by the user itself are stored in /etc/easywall/xml,
			and the modified ports (when the user changes the ports from a program like aMule)
			in /etc/easywall/modifiers.xml.
			
			The active/inactive status for each template is stored in
			/etc/easywall/modifiers.xml """

		self.ports=[]
		self.active=[]
		self.read_port_files("/etc/easywall/xml",self.ports,False)
		for element in self.ports:
			element["category"]="user_defined"
		self.read_port_files("/usr/share/easywall/xml",self.ports,True)
		self.read_port_files("/usr/local/share/easywall/xml",self.ports,True)
		
		ractive=easywall_read_active()
		self.active=ractive.active[:]
		for element in self.active:
			for element2 in self.ports:
				if element2["unique-id"]==element:
					element2["active"]=True

		rmodified=easywall_read_modifiers()
		self.modifiers=rmodified.modifiers


	def read_port_files(self,path,ports,remove):
		
		""" This method reads all the files in PATH and returns all the data in a list """
		
		reader=easywall_read_xml()
		
		try:
			dir=os.listdir(path)
		except:
			return

		for element in dir:
			if (element[-1]=="~"):
				continue
			filename=os.path.join(path,element)
			reader.read_xml(filename)
			if (reader.current_program!=None):
				if len(reader.current_program["files"])==0:
					do_remove=False
				else:
					do_remove=True
					for element in reader.current_program["files"]:
						if (os.path.exists(element)):
							do_remove=False
							break
				reader.current_program["remove"]=do_remove
				if (remove==False) or (do_remove==False):
					# append only if:
					# we want to append all (remove=False)
					# there are no conditional files
					# or there are conditional files, and at least one has been found
					ports.append(reader.current_program)

