#!/usr/bin/env python
# -*- coding: UTF-8 -*-

# Copyright 2008 (C) Raster Software Vigo (Sergio Costas)

# This file is part of EasyWall
#
# EasyWall is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# EasyWall is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# this file contains the interface with PyGTK

import pygtk # for testing GTK version number
pygtk.require ('2.0')
import gtk
import gtk.glade
import locale
import gettext
import os
import gobject
import subprocess

import easywall_core

class show_error:
	
	""" Shows a window with an error """
	
	def __init__(self,title,text,extra,share_path):
		
		if share_path!=".":
			gladepath=os.path.join(share_path,"easywall","easywall.glade")
		else:
			gladepath=os.path.join(share_path,"easywall.glade")
		
		wtree=gtk.glade.XML(gladepath,"show_error",domain="easywall")
		mainwindow=wtree.get_widget("show_error")
		label=wtree.get_widget("error_label")
		label.set_text(text)
		base_text=wtree.get_widget("error_text")
		base_buffer=gtk.TextBuffer()
		base_buffer.set_text(extra)
		base_text.set_buffer(base_buffer)
		mainwindow.set_title(title)
		mainwindow.show()
		mainwindow.run()
		mainwindow.hide()
		mainwindow.destroy()
		mainwindow=None
		label=None
		wtree=None


class ask_exit:
	
	""" Asks if the user wants to exit. Returns True for Yes and False for No """
	
	def run(self,share_path):
		
		if share_path!=".":
			gladepath=os.path.join(share_path,"easywall","easywall.glade")
		else:
			gladepath=os.path.join(share_path,"easywall.glade")
		
		wtree=gtk.glade.XML(gladepath,"ask_exit",domain="easywall")
		mainwindow=wtree.get_widget("ask_exit")
		mainwindow.show()
		retval=mainwindow.run()
		mainwindow.hide()
		mainwindow.destroy()
		mainwindow=None
		wtree=None
		if retval==-8:
			return True
		else:
			return False


class show_ports:
	
	""" Shows (and allows to change) an application's ports """
	
	def __init__(self,program_id,easywall_object,share_path,update_button):
		
		self.update_button=update_button
		self.easywall=easywall_object
		self.program_id=program_id
		if share_path!=".":
			gladepath=os.path.join(share_path,"easywall","easywall.glade")
		else:
			gladepath=os.path.join(share_path,"easywall.glade")
		
		self.wtree=gtk.glade.XML(gladepath,"show_ports",domain="easywall")
		self.wtree.signal_autoconnect(self)
		self.mainwindow=self.wtree.get_widget("show_ports")
		self.mainwindow.show()
		
		self.container=self.wtree.get_widget("port_list")
		label=self.wtree.get_widget("program_name")
		progs=self.easywall.get_program_list()
		for element in progs:
			if element["unique-id"]==program_id:
				label.set_markup("<b>"+element["name"]+"</b>")
				self.mainwindow.set_title("EasyWall ("+element["name"]+")")
		
		
		self.fill_ports(True)


	def fill_ports(self,generate):
		
		self.port_list=self.easywall.get_program_ports(self.program_id)
		
		text1=_("TCP ports from %(from)d to %(to)d")
		text2=_("UDP ports from %(from)d to %(to)d")
		
		p1=text1.find("%(from)d")
		p2=text1.find("%(to)d")
		t1_tcp=text1[:p1]
		t2_tcp=text1[p1+8:p2]
		t3_tcp=text1[p2+6:]
		
		p1=text2.find("%(from)d")
		p2=text2.find("%(to)d")
		t1_udp=text2[:p1]
		t2_udp=text2[p1+8:p2]
		t3_udp=text2[p2+6:]
		
		if generate:
			vport=gtk.Viewport()
			vport.show()
			box=gtk.VBox(False,4)#Table(len(self.port_list),5)
			box.show()
			vport.add(box)
			vport.set_shadow_type(gtk.SHADOW_NONE)
			self.container.add(vport)
			self.labels=[]
			self.old_values={}

		pos=0
		pos2=0
		for element in self.port_list:
			if generate:
				table=gtk.HBox()
				table.show()
				box.pack_start(table,False,True)
				if (element["range"]<=1) or (element["icmp"]==True):
					if element["name"]==None:
						if element["icmp"]:
							label=gtk.Label(_("ICMP message:")+" ")
						elif element["tcp"]:
							label=gtk.Label(_("TCP port:")+" ")
						else:
							label=gtk.Label(_("UDP port:")+" ")
					else:
						label=gtk.Label(element["name"]+" ")
					label.show()
					label.set_justify(gtk.JUSTIFY_RIGHT)
					label.set_alignment(1,0.5)
					table.pack_start(label,True,True)
					if (element["editable"]) and (element["id"]!=""):
						edit=gtk.SpinButton()
						edit.set_digits(0)
						edit.set_increments(1,10)
						table.pack_start(edit,False,True)
						edit.show()
						edit.set_range(1,65535)
						edit.set_value(element["value"])
						self.old_values[element["id"]]=[element["value"],element["range"]]
						edit.connect("value-changed",self.edit_changed,element["id"])
					else:
						label2=gtk.Label("")
						label2.show()
						label2.set_alignment(0,0.5)
						label2.set_justify(gtk.JUSTIFY_LEFT)
						label2.set_alignment(0,0.5)
						if (element["icmp"]):
							if element["name"]==None:
								if element["icmp_code"]==-1:
									text=str(element["icmp_type"])
								else:
									text=str(element["icmp_type"])+"/"+str(element["icmp_code"])
								label2.set_text(text)
						else:
							self.labels.append(label2)
						table.pack_start(label2,False,True)
					label_white=gtk.Label(" ")
					label_white.show()
					table.pack_start(label_white,True,True)
				else:
					if element["tcp"]:
						t1=t1_tcp
						t2=t2_tcp
						t3=t3_tcp
					else:
						t1=t1_udp
						t2=t2_udp
						t3=t3_udp

					if (element["editable"]) and (element["id"]!=""):
						label1=gtk.Label(t1)
						label2=gtk.Label(t2)
						label3=gtk.Label(t3)
						label1.set_justify(gtk.JUSTIFY_RIGHT)
						label1.set_alignment(1,0.5)
						label3.set_justify(gtk.JUSTIFY_LEFT)
						label3.set_alignment(0,0.5)
						edit1=gtk.SpinButton()
						edit1.set_digits(0)
						edit1.set_increments(1,10)
						edit2=gtk.SpinButton()
						edit2.set_digits(0)
						edit2.set_increments(1,10)
						table.pack_start(label1,True,True)
						table.pack_start(edit1,False,True)
						table.pack_start(label2,False,True)
						table.pack_start(edit2,False,True)
						table.pack_start(label3,True,True)
						label1.show()
						label2.show()
						label3.show()
						val1=element["value"]
						val2=element["value"]+element["range"]-1
						if val1>val2:
							tmp=val1
							val1=val2
							val2=tmp
						edit1.show()
						edit1.set_range(1,65535)
						edit1.set_value(val1)
						edit2.show()
						edit2.set_range(val1,65535)
						edit2.set_value(val2)
						self.old_values[element["id"]]=[element["value"],element["range"]]
						edit1.connect("value-changed",self.edit_changed_1,element["id"],edit2)
						edit2.connect("value-changed",self.edit_changed_2,element["id"],edit1)
					else:
						if element["tcp"]:
							label=gtk.Label((_("TCP ports from %(from)d to %(to)d")) % {"from":element["value"],"to":element["value"]+element["range"]-1})
						else:
							label=gtk.Label((_("UDP ports from %(from)d to %(to)d")) % {"from":element["value"],"to":element["value"]+element["range"]-1})
						label.show()
						table.pack_start(label,True,True)
				
			if (element["range"]<=1) and (element["editable"]==False) and (element["icmp"]==False):
				self.labels[pos2].set_text(str(element["value"]))
				pos2+=1
			pos+=1

		if generate:
			modules=self.easywall.get_program_modules(self.program_id)
			for element in modules:
				table=gtk.HBox()
				table.show()
				box.pack_start(table,False,True)
				label=gtk.Label((_("Load module: %(module_name)s"))%{"module_name":element})
				label.show()
				table.pack_start(label,True,True)

			table=gtk.HBox()
			table.show()
			box.pack_start(table,True,True)
					
			
	def edit_changed(self,widget,value=None):

		if value==None:
			return

		self.easywall.set_port_value(self.program_id,value,int(widget.get_value()),-1)
		self.fill_ports(False)


	def edit_changed_1(self,widget1,value=None,widget2=None):

		if (value==None) or (widget2==None):
			return

		v1=int(widget1.get_value())
		v2=int(widget2.get_value())
		
		widget2.set_range(v1,65535)
		
		if v1>v2:
			v2=v1
			widget2.set_value(v2)

		self.easywall.set_port_value(self.program_id,value,v1,v2-v1+1)
		self.fill_ports(False)


	def edit_changed_2(self,widget2,value=None,widget1=None):

		if (value==None) or (widget1==None):
			return

		v1=int(widget1.get_value())
		v2=int(widget2.get_value())
		
		if v1>v2:
			v1=v2
			widget1.set_value(v1)

		self.easywall.set_port_value(self.program_id,value,v1,v2-v1+1)
		self.fill_ports(False)


	def on_port_cancel_clicked(self,widget,extra=None):
		
		for element in self.old_values:
			self.easywall.set_port_value(self.program_id,element,self.old_values[element][0],self.old_values[element][1])
		
		if extra==None:
			self.mainwindow.hide()
			self.mainwindow.destroy()
			self.mainwindow=None


	def on_port_accept_clicked(self,widget):

		port_list=self.easywall.get_program_ports(self.program_id)
		for element in port_list:
			if element["id"]=="":
				continue
			if self.old_values.has_key(element["id"])==False:
				continue
			if element["value"]!=self.old_values[element["id"]]:
				self.update_button()
				
		self.mainwindow.hide()
		self.mainwindow.destroy()
		self.mainwindow=None


class udport_add:
	
	def __init__(self,gladepath):
		
		self.newtree=gtk.glade.XML(gladepath,"add_udport",domain="easywall")
		self.newtree.signal_autoconnect(self)
		self.newwindow=self.newtree.get_widget("add_udport")
		self.first_port=self.newtree.get_widget("first_port")
		self.last_port=self.newtree.get_widget("last_port")
		self.value_entry=self.newtree.get_widget("port_number")
		self.on_single_radio_toggled(self.newtree.get_widget("single_radio"))
		self.newwindow.show()


	def on_single_radio_toggled(self,widget):
		
		if widget.get_active():
			self.value_entry.set_sensitive(True)
			self.first_port.set_sensitive(False)
			self.last_port.set_sensitive(False)
		else:
			self.value_entry.set_sensitive(False)
			self.first_port.set_sensitive(True)
			self.last_port.set_sensitive(True)


	def on_first_port_value_changed(self,widget):
		
		v1=self.first_port.get_value_as_int()
		v2=self.last_port.get_value_as_int()
		
		if v2<v1:
			v2=v1
			self.last_port.set_value(v1)
		self.last_port.set_range(v1,65535)

		
	def run(self):
		
		self.tcp_ports=[]
		self.udp_ports=[]
		
		retval=self.newwindow.run()
		if retval==-6: # accept clicked
			istcp=self.newtree.get_widget("type_tcp").get_active()
			single=self.newtree.get_widget("single_radio").get_active()
			if single:
				value1=self.newtree.get_widget("port_number").get_value_as_int()
				value2=value1
			else:
				value1=self.newtree.get_widget("first_port").get_value_as_int()
				value2=self.newtree.get_widget("last_port").get_value_as_int()

			for counter in range(value2-value1+1):
				value=value1+counter
				if istcp:
					self.tcp_ports.append(value)
				else:
					self.udp_ports.append(value)

		self.newwindow.hide()
		self.newwindow.destroy()
		self.newwindow=None
		self.newtree=None

		return self.tcp_ports,self.udp_ports


class show_udports:
	
	def __init__(self,uid,wall_class,share_path,callback):
		
		self.uid=uid
		self.easywall=wall_class
		self.callback=callback

		if share_path!=".":
			self.gladepath=os.path.join(share_path,"easywall","easywall.glade")
		else:
			self.gladepath=os.path.join(share_path,"easywall.glade")
		
		self.wtree=gtk.glade.XML(self.gladepath,"show_udports",domain="easywall")
		self.wtree.signal_autoconnect(self)
		self.mainwindow=self.wtree.get_widget("show_udports")
		self.mainwindow.show()
		self.accept_button=self.wtree.get_widget("udport_accept")
		self.wtext=self.wtree.get_widget("program_name_entry")
		
		self.treeview=self.wtree.get_widget("udport_list")
		self.model=gtk.ListStore(gobject.TYPE_INT,gobject.TYPE_INT,gobject.TYPE_BOOLEAN,gobject.TYPE_STRING)
		self.treeview.set_model(self.model)
		render1=gtk.CellRendererText()
		column1=gtk.TreeViewColumn(_("Port"),render1,text=3)
		column1.set_expand(True)
		self.treeview.append_column(column1)
		self.ports_tcp=[65536]
		self.ports_udp=[65536]
		if self.uid!=None:
			self.get_ports()
			data=self.easywall.get_program_info(uid)
			if data!=None:
				self.wtext.set_text(data["name"])
		self.refresh_port_list()
		self.set_accept()
			
	
	def get_ports(self):
		
		self.ports_tcp=[65536]
		self.ports_udp=[65536]
		lports=self.easywall.get_program_ports(self.uid)
		for element in lports:
			if element["range"]<2:
				if element["tcp"]:
					if self.ports_tcp.count(element["value"])==0:
						self.ports_tcp.append(element["value"])
				else:
					if self.ports_udp.count(element["value"])==0:
						self.ports_udp.append(element["value"])
			else:
				for val in range(element["range"]):
					if element["tcp"]:
						if self.ports_tcp.count(element["value"]+val)==0:
							self.ports_tcp.append(element["value"]+val)
					else:
						if self.ports_udp.count(element["value"]+val)==0:
							self.ports_udp.append(element["value"]+val)

		self.ports_tcp.sort()
		self.ports_udp.sort()


	def on_udport_del_clicked(self,widget):
		
		selection,iterator = self.treeview.get_selection().get_selected()
		if iterator==None:
			return
		
		first=selection.get_value(iterator,0)
		last=selection.get_value(iterator,1)
		istcp=selection.get_value(iterator,2)
		
		newports=[]
		if istcp:
			for element in self.ports_tcp[:-1]:
				if (element<first) or (element>(last+first-1)):
					newports.append(element)
			self.ports_tcp=newports
			self.ports_tcp.append(65536)
			self.ports_tcp.sort()
		else:
			for element in self.ports_udp[:-1]:
				if (element<first) or (element>(last+first-1)):
					newports.append(element)
			self.ports_udp=newports
			self.ports_udp.append(65536)
			self.ports_udp.sort()
			
		self.refresh_port_list()


	def on_udport_add_clicked(self,widget):
		
		newwin=udport_add(self.gladepath)
		tcp_ports,udp_ports=newwin.run()
		newwin=None
		
		for element in tcp_ports:
			if self.ports_tcp.count(element)==0:
				self.ports_tcp.append(element)
		self.ports_tcp.sort()

		for element in udp_ports:
			if self.ports_udp.count(element)==0:
				self.ports_udp.append(element)
		self.ports_udp.sort()
		
		self.refresh_port_list()


	def on_udport_cancel_clicked(self,widget):
		
		self.mainwindow.hide()
		self.mainwindow.destroy()
		
	
	def on_udport_accept_clicked(self,widget):
		
		name=self.wtext.get_text()
		port_list=[]
		for element in self.ports_tcp[:-1]:
			port_list.append((element,True))
		for element in self.ports_udp[:-1]:
			port_list.append((element,False))
		self.easywall.change_user_defined_program(self.uid,name,port_list)
		self.callback()
		self.mainwindow.hide()
		self.mainwindow.destroy()


	def on_program_name_changed(self,widget):

		self.set_accept()


	def set_accept(self):

		value=True
		text=self.wtext.get_text()
		if len(text)<1:
			value=False
		
		if (len(self.ports_tcp)==1) and (len(self.ports_udp)==1):
			value=False
		
		self.accept_button.set_sensitive(value)


	def refresh_port_list(self):
		
		self.model.clear()
		pos=0
		loop=False

		for element in self.ports_tcp:
			if loop==False:
				first_element=element
				next_element=element+1
				count=1
				loop=True
			else:
				if element==next_element:
					count+=1
					next_element+=1
				else:
					iterator=self.model.insert(pos)
					self.model.set_value(iterator,0,first_element)
					self.model.set_value(iterator,1,count)
					self.model.set_value(iterator,2,True)
					if count==1:
						self.model.set_value(iterator,3,_("TCP port:")+" "+str(first_element))
					else:
						self.model.set_value(iterator,3,_("TCP ports from %(from)d to %(to)d")%{"from":first_element,"to":next_element-1})
					first_element=element
					next_element=element+1
					count=1
					pos+=1

		loop=False
		for element in self.ports_udp:
			if loop==False:
				first_element=element
				next_element=element+1
				count=1
				loop=True
			else:
				if element==next_element:
					count+=1
					next_element+=1
				else:
					iterator=self.model.insert(pos)
					self.model.set_value(iterator,0,first_element)
					self.model.set_value(iterator,1,count)
					self.model.set_value(iterator,2,False)
					if count==1:
						self.model.set_value(iterator,3,_("UDP port:")+" "+str(first_element))
					else:
						self.model.set_value(iterator,3,_("UDP ports from %(from)d to %(to)d")%{"from":first_element,"to":next_element-1})
					first_element=element
					next_element=element+1
					count=1
					pos+=1

		self.set_accept()


class preferences:
	
	def __init__(self,share_path,easywall_class):
		
		self.easywall=easywall_class
		
		if share_path!=".":
			self.gladepath=os.path.join(share_path,"easywall","easywall.glade")
		else:
			self.gladepath=os.path.join(share_path,"easywall.glade")
		
		self.wtree=gtk.glade.XML(self.gladepath,"preferences_window",domain="easywall")
		self.wtree.signal_autoconnect(self)
		self.mainwindow=self.wtree.get_widget("preferences_window")
		self.base_text=self.wtree.get_widget("base_txt")
		self.base_text.set_accepts_tab(True)
		self.base_buffer=gtk.TextBuffer()
		self.base_text.set_buffer(self.base_buffer)
		self.line_text=self.wtree.get_widget("line_template")
		self.icmp_text=self.wtree.get_widget("icmp_template")
		self.module_text=self.wtree.get_widget("load_module_template")
		
		self.template_base,self.template_line,self.template_icmp,self.template_module=self.easywall.get_templates()
		self.show_templates()
		

	def run(self):
		
		self.mainwindow.show()
		while(True):
			retval=self.mainwindow.run()
			if retval!=1:
				break
			else:
				self.easywall.restore_templates()
				self.show_templates()

		if retval!=-6:
			self.easywall.set_templates(self.template_base,self.template_line,self.template_icmp,self.template_module)
			self.easywall.write_template()
			self.mainwindow.hide()
			self.mainwindow.destroy()
			return 0 # not changed
		
		template_base=self.base_buffer.get_text(self.base_buffer.get_start_iter(),self.base_buffer.get_end_iter())
		template_line=self.line_text.get_text()
		template_icmp=self.icmp_text.get_text()
		template_module=self.module_text.get_text()
		
		self.easywall.set_templates(template_base,template_line,template_icmp,template_module)
		self.easywall.write_template()
		
		self.mainwindow.hide()
		self.mainwindow.destroy()
		return 1 # changed
	
	
	def show_templates(self):
		
		template_base,template_line,template_icmp,template_module=self.easywall.get_templates()
		
		self.base_buffer.set_text(template_base)
		self.line_text.set_text(template_line)
		self.icmp_text.set_text(template_icmp)
		self.module_text.set_text(template_module)
		

class easywall_gtk:
	
	def __init__(self,share_path,share_locale,help_path):
		
		self.share_path=share_path
		self.share_locale=share_locale
		self.help_path=help_path
		if share_path!=".":
			self.gladepath=os.path.join(share_path,"easywall","easywall.glade")
		else:
			self.gladepath=os.path.join(share_path,"easywall.glade")
		
		self.easywall=easywall_core.easywall_core(locale.getdefaultlocale()[0])

		self.tabs=None
		self.modified=False
		
		
	def set_locale(self):
		
		""" Init the GETTEXT functions for GTK """
		
		gtk.glade.bindtextdomain("easywall",self.share_locale)


	def page_changed(self,notebook,page,page_num):
		
		button1=self.wtree.get_widget("add_program")
		button2=self.wtree.get_widget("del_program")
		
		if self.types[page_num]==self.ud_category:
			button2.set_sensitive(True)
		else:
			button2.set_sensitive(False)


	def refresh_programs(self,show_last=False):
		
		if self.tabs!=None:
			self.tabs.destroy()

		self.types_tmp=[]
		programs=self.easywall.get_program_list()
		self.ud_category=self.easywall.get_user_defined_category()
		for element in programs:
			if (self.types_tmp.count(element["category"])==0) and (element["category"]!=self.ud_category):
				self.types_tmp.append(element["category"])

		self.types_tmp.sort()
		
		# add the User Defined category at the begining, always
		self.types=[]
		for element in self.types_tmp:
			self.types.append(element)
		self.types.append(self.ud_category)
			
		self.tabs=gtk.Notebook()
		self.tabs.set_tab_pos(gtk.POS_LEFT)
		self.tabs.connect("switch-page",self.page_changed)
		self.wtree.get_widget("main_container").pack_start(self.tabs)
		self.tabs.show()
		
		# add the list of elements
		
		self.treeviews=[]
		for element in self.types:
			label=gtk.Label(element)
			container=gtk.ScrolledWindow()
			container.set_border_width(3)
			container.set_policy(gtk.POLICY_AUTOMATIC,gtk.POLICY_AUTOMATIC)
			container.show()
			label.show()
			self.tabs.append_page(container,label)
			
			treeview=gtk.TreeView()
			treeview.connect("button-press-event",self.on_program_pressed)
			self.treeviews.append(treeview)
			treeview.show()
			container.add(treeview)
			
			# the ListStore contains:
			# * the unique-id
			# * the translated name
			# * the status (active or inactive)
			
			model=gtk.ListStore(gobject.TYPE_STRING,gobject.TYPE_STRING,gobject.TYPE_BOOLEAN,gobject.TYPE_STRING)
			
			treeview.set_model(model)
			render1=gtk.CellRendererText()
			column1=gtk.TreeViewColumn(_("Program"),render1,text=1,foreground=3)
			column1.set_expand(True)
			render2=gtk.CellRendererToggle()
			render2.set_property("activatable",True)
			column2=gtk.TreeViewColumn(_("Status"),render2,active=2)
			column2.set_expand(False)
			render2.connect("toggled", self.toggled_cb, model)
			treeview.append_column(column2)
			treeview.append_column(column1)
			
			pos=0
			for element2 in programs:
				if element2["category"]==element:
					ports=self.easywall.get_program_ports(element2["unique-id"])
					color="#000000"
					for element3 in ports:
						if element3["editable"]:
							color="#0000FF"
							break
					iterator=model.insert(pos)
					model.set_value(iterator,0,element2["unique-id"])
					model.set_value(iterator,1,element2["name"])
					model.set_value(iterator,2,element2["active"])
					model.set_value(iterator,3,color)
					pos+=1
					
			if (show_last):
				self.tabs.set_current_page(self.tabs.get_n_pages()-1)

			
	def toggled_cb(self,cell, path, model):
		
		model[path][2] = not model[path][2]
		self.easywall.set_program_status(model[path][0],model[path][2])
		self.change_done()
		
		
	def change_done(self):
		self.apply_changes.set_sensitive(True)
		self.modified=True

	
	def on_menu_preferences(self,widget):
		
		prefs=preferences(self.share_path,self.easywall)
		retval=prefs.run()
		if retval==1: # firewall changed
			self.on_apply_changes_clicked(None)
	
	
	def on_apply_changes_clicked(self,widget):
		
		self.easywall.store_configuration()
		self.easywall.create_firewall()
		if self.easywall.get_firewall_status():
			retval=self.easywall.set_firewall_status("restart")
			if (retval!=0) and (retval!=None):
				w=show_error(_("Error while updating the firewall"),_("Failed to restart the firewall.\nCheck that the templates are correct."),self.easywall.get_output(),self.share_path)
				w=None
				self.easywall.set_firewall_status("stop")
		self.refresh_firewall_status()
		self.apply_changes.set_sensitive(False)
		self.modified=False


	def refresh_firewall_status(self):
		
		w1=self.wtree.get_widget("start_firewall")
		w2=self.wtree.get_widget("stop_firewall")
		
		status=self.easywall.get_firewall_install()
		if status==False: # the firewall isn't still installed
			self.easywall.create_firewall()

		status=self.easywall.get_firewall_status()
		if status: # the firewall is active
			w1.hide()
			w2.show()
		else:
			w1.show()
			w2.hide()
			
		
	def on_stop_clicked(self,widget):
		
		retval=self.easywall.set_firewall_status("stop")

		if (retval!=0) and (retval!=None):
			w=show_error(_("Error stoping the firewall"),_("Failed to stop the firewall.\nCheck that the templates are correct."),self.easywall.get_output(),self.share_path)
			w=None
		self.refresh_firewall_status()
		

	def on_start_clicked(self,widget):
		
		retval=self.easywall.set_firewall_status("start")
		
		if (retval!=0) and (retval!=None):
			w=show_error(_("Error starting the firewall"),_("Failed to start the firewall.\nCheck that the templates are correct."),self.easywall.get_output(),self.share_path)
			w=None
			self.easywall.set_firewall_status("stop")
		self.refresh_firewall_status()


	def added_program(self):
		self.refresh_programs(True)
		self.change_done()


	def on_add_program_clicked(self,widget):
		
		self.tabs.set_current_page(self.tabs.get_n_pages()-1)
		show=show_udports(None,self.easywall,self.share_path,self.added_program)


	def on_del_program_clicked(self,widget):
		
		current_treeview=self.treeviews[self.tabs.get_current_page()]
		selection,iterator=current_treeview.get_selection().get_selected()
		if iterator==None:
			return # no selected programcurrent_treeview=self.treeviews[self.tabs.get_current_page()]
		
		newtree=gtk.glade.XML(self.gladepath,"ask_remove_program",domain="easywall")
		newwindow=newtree.get_widget("ask_remove_program")
		label=newtree.get_widget("ud_program_name")
		label.set_text(selection.get_value(iterator,1))
		newwindow.show()
		retval=newwindow.run()
		if retval==-6: # YES clicked
			self.easywall.remove_user_defined_program(selection.get_value(iterator,0))
			self.added_program()
		newwindow.hide()
		newwindow.destroy()
		newwindow=None
		newtree=None


	def on_program_pressed(self,widget,event):

		if event.type != gtk.gdk._2BUTTON_PRESS:
			return

		self.on_ports_clicked(None)


	def on_ports_clicked(self,widget):
		
		current_treeview=self.treeviews[self.tabs.get_current_page()]
		selection,iterator=current_treeview.get_selection().get_selected()
		if iterator==None:
			return # no selected programcurrent_treeview=self.treeviews[self.tabs.get_current_page()]
		
		page_num=self.tabs.get_current_page()
		if self.types[page_num]==self.ud_category:
			show=show_udports(selection.get_value(iterator,0),self.easywall,self.share_path,self.added_program)
		else:
			show=show_ports(selection.get_value(iterator,0),self.easywall,self.share_path,self.change_done)


	def on_cancel_clicked(self,widget,more=None):
		
		if self.modified:
			ask=ask_exit()
			retval=ask.run(self.share_path)
			if retval:
				gtk.main_quit()
			else:
				return True

		gtk.main_quit()


	def on_menu_about(self,widget):
		
		tree=gtk.glade.XML(self.gladepath,"about",domain="easywall")
		wabout=tree.get_widget("about")
		wabout.show()
		wabout.run()
		wabout.hide()
		wabout.destroy()
		wabout=None
		tree=None


	def on_menu_help(self,widget):
		
		file=os.path.join(self.help_path,"html","index.html")
		
		launch_list=[[False,"epiphany"],[False,"konqueror"],[False,"firefox -new-window"],[False,"opera -newwindow"]]
		for program in launch_list:
			if program[0]:
				cprogram=program[1]+" file://"+file
			else:
				cprogram=program[1]+" "+file
			retval=subprocess.Popen(cprogram,shell=True).wait()
			print cprogram+" "+str(retval)
			if retval==0:
				break

		if retval!=0:
			print "Can't show help"
			

	def show_main(self):
		
		try:
			f=open("/etc/easywall/showed_note","r")
			f.close()
		except:
			wt=gtk.glade.XML(self.gladepath,"warning_dialog",domain="easywall")
			mw=wt.get_widget("warning_dialog")
			mw.show()
			mw.run()
			mw.destroy()
		try:
			f=open("/etc/easywall/showed_note","w")
			f.write("done")
			f.close()
		except:
			pass
		
		self.wtree=gtk.glade.XML(self.gladepath,"mainwindow",domain="easywall")
		self.mainwindow=self.wtree.get_widget("mainwindow")
		self.wtree.signal_autoconnect(self)
		self.apply_changes=self.wtree.get_widget("apply_changes")
		self.apply_changes.set_sensitive(False)
		self.refresh_programs()
		self.refresh_firewall_status()
		self.mainwindow.show()
		gtk.main()

