#!/bin/bash

#	Get script directory path.

scriptdir="`dirname ${0}`"
DESTDIR="${DESTDIR:-}"


install_lang()		# arg1=datadir, arg2=locale, arg3=<lang file name>.

{
	install -m 755 -d "${DESTDIR}${1}/locale/${2}/LC_MESSAGES"
	install -m 644 "${scriptdir}/po/${3}.mo"			\
		"${DESTDIR}${1}/locale/${2}/LC_MESSAGES/easywall.mo"
}


install_locales()	# arg1=datadir.

{
	install_lang "${1}"	es	es		# Spanish
	install_lang "${1}"	gl	gl		# Galego
}


install_others()	# arg1=bindir, arg2=datadir, arg3=pkglibdir,
			#	arg4=pkgdatadir, arg5=pkgdocdir.

{
	install -m 755 -d /etc/easywall/xml

	install -m 755 -d "${DESTDIR}${1}"
	install -m 755 "${scriptdir}/easywall.py" "${DESTDIR}${1}/easywall"

	install -m 755 -d "${DESTDIR}${3}"
	install "${scriptdir}"/easywall_*.py "${DESTDIR}${3}/"

	install -m 755 -d "${DESTDIR}${4}"
	install -m 644 "${scriptdir}/easywall.glade" "${DESTDIR}${4}/"
	install -m 644 "${scriptdir}/easywall.svg" "${DESTDIR}${4}/"

	install -m 755 -d "${DESTDIR}${2}/pixmaps"
	install -m 644 "${scriptdir}/easywall.svg" "${DESTDIR}${2}/pixmaps/"

	install -m 755 -d "${DESTDIR}${2}/applications"
	install -m 644 "${scriptdir}/easywall.desktop" "${DESTDIR}${2}/applications/"

	install -m 755 -d "${DESTDIR}${5}"
	install -m 644 "${scriptdir}/docs"/* "${DESTDIR}${5}" 2>/dev/null

	install -m 755 -d "${DESTDIR}${5}/html"
	install -m 644 "${scriptdir}/docs/html"/* "${DESTDIR}${5}/html"

	install -m 755 -d "${DESTDIR}${4}/xml"
	install -m 644 "${scriptdir}/xml"/* "${DESTDIR}${4}/xml"

	install -m 755 -d /etc/easywall/xml
	install -m 644 "${scriptdir}/templates"/* /etc/easywall

}


#	Process arguments.

PARAM=

for arg
do	if [ "${PARAM}" ]
	then	eval "${PARAM}=\"${arg}\""
		PARAM=
	else
		case "${arg}" in

		--*)	PARAM="${arg: 2}"
			;;

		-*)	PARAM="${arg: 1}"
			;;

		*)	PARAM="${arg}"
			;;
		esac

		case "${PARAM}" in

		*=*)	eval "${PARAM}"
			PARAM=
			;;
		esac
	fi
done

if [ "${PARAM}" ]
then	eval "${PARAM}="
fi

#	Uninstall older versions if needed.

[ "${uninstall}" = "no" ] || "${scriptdir}/uninstall.sh" "${@}"

#	Version is targeted if specified as such, or if a parameter is set.

targeted=${targeted:-${prefix}${bindir}${libdir}${datadir}${docdir}\
${pkglibdir}${pkgdatadir}${pkgdocdir}${DESTDIR}no}

if [ "${targeted}" = "no" ]
then
	#	Version is not targeted. Install new version to the default
	#		paths (relative to DESTDIR).

	install_locales "/usr/share"		# Locales are common.

	install_others	"/usr/local/bin"				\
			"/usr/local/share"				\
			"/usr/local/lib/easywall"				\
			"/usr/local/share/easywall"			\
			"/usr/local/share/doc/easywall"
else

	#	Be sure all paths are defined.

	prefix="${prefix:-/usr/local}"
	bindir="${bindir:-${prefix}/bin}"
	libdir="${libdir:-${prefix}/lib}"
	datadir="${datadir:-${prefix}/share}"
	docdir="${docdir:-${datadir}/doc}"
	pkglibdir="${pkglibdir:-${libdir}/easywall}"
	pkgdatadir="${pkgdatadir:-${datadir}/easywall}"
	pkgdocdir="${pkgdocdir:-${docdir}/easywall}"

	#	And now, install everything according to paths above.

	install_locales "${datadir}"
	install_others	"${bindir}"					\
			"${datadir}"					\
			"${pkglibdir}"					\
			"${pkgdatadir}"					\
			"${pkgdocdir}"
fi

